//
// Simple graph colouring
//  - not DIMACS format 
//  - uses my own dynamic variable ordering heuristic
//
import java.io.File;
import java.io.IOException;
import java.util.Scanner;
import org.chocosolver.solver.*;
import org.chocosolver.solver.variables.*;
import org.chocosolver.solver.constraints.*;
import org.chocosolver.solver.search.strategy.strategy.IntStrategy;
import org.chocosolver.solver.search.strategy.selectors.values.IntDomainMin;
import org.chocosolver.solver.search.strategy.assignments.*;


public class GColMyDVO {

  
    public static void main(String[] args) throws IOException {
	Scanner sc    = new Scanner(new File(args[0])); // graph file, not in DIMACS format
	int n         = sc.nextInt(); // number of vertices
	int m         = Integer.parseInt(args[1]); // number of colours
	Solver solver = new Solver("graph colouring with MyDVO");
	IntVar[] v    = VF.enumeratedArray("v",n,1,m,solver); // n vertices each with m colours

	while (sc.hasNext()){
	    int i = sc.nextInt();
	    int j = sc.nextInt();
	    solver.post(ICF.arithm(v[i],"!=",v[j]));
	}
	sc.close();

	solver.set(new IntStrategy(v,new MyDVO(),new IntDomainMin(),DecisionOperator.int_eq));
	//
	// magic code for using our own dvo
	//

	System.out.println(solver.findSolution());

	for (int i=0;i<n;i++) System.out.print(v[i].getValue() +" ");
	System.out.println();
	System.out.println("nodes: " + solver.getMeasures().getNodeCount() + 
                           "  cpu: " + solver.getMeasures().getTimeCount());
    }
}

